#!/usr/bin/env python3
"""tv_qt_class.py module called by tv_viewer contains a class to handle the PyQt5 GUI of program"""
# =========================MODULE HEADER=======================================
# title             :tv_qt_class
# description       :Module called by tv_viewer contains a class to handle the PyQt5 GUI of program
# author            :Gavin Lyons
# date              :20/11/2017
# web               :https://github.com/gavinlyonsrepo/
# mail              :glyons66@hotmail.com
# python_version    :3.6.0

# ==========================IMPORTS======================
# Import the system modules needed to run tv viewer.
import os
import sys
from PyQt5 import QtCore, QtWidgets

# my modules
from tv_api_work import tv_api_work as myApi
from tv_sqllite import tv_sqllite as mySql
from logger_conf import logger_conf as myLog

# ===================GLOBALS======================================


# setup logging

logger = myLog.my_logging(__name__)


# ===================CLASSES & FUNCTION SECTION===============================


class Ui_MainWindow(object):
    def __init__(self):
        self.ScreenStatus = 0
        # define instance of the API class
        self.myTvApi = myApi.TvApi("myapiobject")
        # define instance of the SQl class
        self.myTvSql = mySql.TvSqLight("mysqlobject")

    def setupUi(self, MainWindow):
        MainWindow.setObjectName("MainWindow")
        MainWindow.resize(1100, 800)
        self.centralwidget = QtWidgets.QWidget(MainWindow)
        self.centralwidget.setObjectName("centralwidget")
        # Buttons
        self.OneBtn = QtWidgets.QPushButton(self.centralwidget)
        self.OneBtn.setGeometry(QtCore.QRect(20, 600, 100, 30))
        self.OneBtn.setObjectName("OneBtn")
        self.TwoBtn = QtWidgets.QPushButton(self.centralwidget)
        self.TwoBtn.setGeometry(QtCore.QRect(140, 600, 100, 30))
        self.TwoBtn.setObjectName("TwoBtn")
        self.ThreeBtn = QtWidgets.QPushButton(self.centralwidget)
        self.ThreeBtn.setGeometry(QtCore.QRect(260, 600, 100, 30))
        self.ThreeBtn.setObjectName("ThreeBtn")
        self.FourBtn = QtWidgets.QPushButton(self.centralwidget)
        self.FourBtn.setGeometry(QtCore.QRect(380, 600, 100, 30))
        self.FourBtn.setObjectName("FourBtn")
        self.FiveBtn = QtWidgets.QPushButton(self.centralwidget)
        self.FiveBtn.setGeometry(QtCore.QRect(500, 600, 100, 30))
        self.FiveBtn.setObjectName("FiveBtn")
        self.ResetBtn = QtWidgets.QPushButton(self.centralwidget)
        self.ResetBtn.setGeometry(QtCore.QRect(620, 600, 100, 30))
        self.ResetBtn.setObjectName("Reset")
        self.ExitBtn = QtWidgets.QPushButton(self.centralwidget)
        self.ExitBtn.setGeometry(QtCore.QRect(740, 600, 100, 30))
        self.ExitBtn.setObjectName("Exit")

        # radio buttons
        self.OverviewRbtn = QtWidgets.QRadioButton(self.centralwidget)
        self.OverviewRbtn.setGeometry(QtCore.QRect(970, 30, 136, 27))
        self.OverviewRbtn.setObjectName("OverviewRbtn")
        self.ActorsRbtn = QtWidgets.QRadioButton(self.centralwidget)
        self.ActorsRbtn.setGeometry(QtCore.QRect(970, 80, 136, 27))
        self.ActorsRbtn.setObjectName("ActorsRbtn")
        self.seasonRbtn = QtWidgets.QRadioButton(self.centralwidget)
        self.seasonRbtn.setGeometry(QtCore.QRect(970, 130, 136, 27))
        self.seasonRbtn.setObjectName("seasonRbtn")
        self.crewsRbtn = QtWidgets.QRadioButton(self.centralwidget)
        self.crewsRbtn.setGeometry(QtCore.QRect(970, 180, 136, 27))
        self.crewsRbtn.setObjectName("crewsRbtn")
        self.infoRbtn = QtWidgets.QRadioButton(self.centralwidget)
        self.infoRbtn.setGeometry(QtCore.QRect(970, 230, 136, 27))
        self.infoRbtn.setObjectName("infoRbtn")
        self.episodeRbtn = QtWidgets.QRadioButton(self.centralwidget)
        self.episodeRbtn.setGeometry(QtCore.QRect(970, 280, 136, 27))
        self.episodeRbtn.setObjectName("episodeRbtn")

        # List box
        self.listinfo = QtWidgets.QListWidget(self.centralwidget)
        self.listinfo.setGeometry(QtCore.QRect(20, 30, 940, 505))
        self.listinfo.setObjectName("listinfo")
        self.listinfo.setWordWrap(True)
        self.listinfo.setProperty("isWrapping", False)
        self.listinfo.setVerticalScrollBarPolicy(QtCore.Qt.ScrollBarAlwaysOn)
        self.listinfo.setVerticalScrollMode(QtWidgets.QAbstractItemView.ScrollPerPixel)
        self.listinfo.setMovement(QtWidgets.QListView.Static)


        # text box
        self.textEnter = QtWidgets.QLineEdit(self.centralwidget)
        self.textEnter.setGeometry(QtCore.QRect(20, 560, 820, 32))
        self.textEnter.setObjectName("textEnter")

        # labels
        self.mainlbl = QtWidgets.QLabel(self.centralwidget)
        self.mainlbl.setGeometry(QtCore.QRect(20, 5, 301, 22))
        self.mainlbl.setObjectName("mainlbl")
        self.txtlbl = QtWidgets.QLabel(self.centralwidget)
        self.txtlbl.setGeometry(QtCore.QRect(20, 535, 90, 22))
        self.txtlbl.setObjectName("txtlbl")
        MainWindow.setCentralWidget(self.centralwidget)

        # menubar
        self.menubar = QtWidgets.QMenuBar(MainWindow)
        self.menubar.setGeometry(QtCore.QRect(0, 0, 800, 30))
        self.menubar.setObjectName("menubar")
        self.menuFile = QtWidgets.QMenu(self.menubar)
        self.menuFile.setObjectName("menuFile")
        MainWindow.setMenuBar(self.menubar)

        self.statusbar = QtWidgets.QStatusBar(MainWindow)
        self.statusbar.setObjectName("statusbar")
        MainWindow.setStatusBar(self.statusbar)

        # add action to menus
        self.actionAbout = QtWidgets.QAction(MainWindow)
        self.actionAbout.setObjectName("actionAbout")
        self.actionHelp = QtWidgets.QAction(MainWindow)
        self.actionHelp.setObjectName("actionHelp")
        self.actionExit = QtWidgets.QAction(MainWindow)
        self.actionExit.setObjectName("actionExit")
        self.menuFile.addAction(self.actionAbout)
        self.menuFile.addAction(self.actionHelp)
        self.menuFile.addAction(self.actionExit)
        self.menubar.addAction(self.menuFile.menuAction())

        self.retranslateUi(MainWindow)
        QtCore.QMetaObject.connectSlotsByName(MainWindow)

    def retranslateUi(self, MainWindow):
        _translate = QtCore.QCoreApplication.translate
        MainWindow.setWindowTitle(_translate("MainWindow", "Tv Viewer"))

        # Buttons
        self.OneBtn.setText(_translate("MainWindow", "Enter"))
        self.OneBtn.clicked.connect(self.one_button_click)

        self.TwoBtn.setText(_translate("MainWindow", "Select"))
        self.TwoBtn.setEnabled(False)
        self.TwoBtn.clicked.connect(self.two_button_click)

        self.ThreeBtn.setText(_translate("MainWindow", "Upcoming"))
        self.ThreeBtn.clicked.connect(self.three_button_click)
        self.ThreeBtn.setEnabled(False)

        self.FourBtn.setText(_translate("MainWindow", "View Favs"))
        self.FourBtn.clicked.connect(self.four_button_click)

        self.FiveBtn.setText(_translate("MainWindow", "Edit Favs"))
        self.FiveBtn.clicked.connect(self.five_button_click)
        self.FiveBtn.setEnabled(False)

        self.ResetBtn.setText(_translate("MainWindow", "Reset"))
        self.ResetBtn.clicked.connect(self.reset_screen)

        self.ExitBtn.setText(_translate("MainWindow", "Exit"))
        self.ExitBtn.clicked.connect(self.close_application)

        # labels
        self.mainlbl.setText(_translate("MainWindow", "Information Display"))
        self.txtlbl.setText(_translate("MainWindow", "Input Box"))

        # Menubar events
        self.menuFile.setTitle(_translate("MainWindow", "Menu"))
        # about
        self.actionAbout.setText(_translate("MainWindow", "About"))
        self.actionAbout.setShortcut("Ctrl+A")
        self.actionAbout.setStatusTip('App info')
        self.actionAbout.triggered.connect(self.display_about)
        # help menu item
        self.actionHelp.setText(_translate("MainWindow", "Help"))
        self.actionHelp.setShortcut("Ctrl+H")
        self.actionHelp.setStatusTip('Display The Help')
        self.actionHelp.triggered.connect(self.display_help)
        # exit menu item
        self.actionExit.setText(_translate("MainWindow", "&Exit"))
        self.actionExit.setShortcut("Ctrl+Q")
        self.actionExit.setStatusTip('Leave The App')
        self.actionExit.triggered.connect(self.close_application)

        # radio buttons
        self.OverviewRbtn.setText(_translate("MainWindow", "Overview"))
        self.ActorsRbtn.setText(_translate("MainWindow", "Actors"))
        self.seasonRbtn.setText(_translate("MainWindow", "Seasons"))
        self.crewsRbtn.setText(_translate("MainWindow", "Crews"))
        self.infoRbtn.setText(_translate("MainWindow", "More Info"))
        self.episodeRbtn.setText(_translate("MainWindow", "Episodes"))

        # initialise screen
        self.start_screen()

        # create the database
        self.myTvSql.create_db()



    def close_application(self):
        """ Method control exit of program on exit button click closes database"""
        self.myTvSql.close_db()
        logger.info(' Program Exiting ')
        sys.exit()

    def display_help(self):
        """ Method to display about information on menubar about click"""
        self.listinfo.clear()
        self.listinfo.addItem("\n Readme(linux only) at :- \
                               \n /usr/share/doc/tv_viewer/\
                               \n or\
                               \n https://github.com/gavinlyonsrepo/tv_viewer")

    def display_about(self):
        """ Method to display about information on menubar help click"""
        self.listinfo.clear()
        self.listinfo.addItem("\n GUI program to view tv program details using tvmaze API \
                               \n Version 1.1 \
                               \n Copyright Gavin Lyons 2017 GPL \
                               \n glyons66@hotmail.com \
                               \n https://github.com/gavinlyonsrepo/tv_viewer \
                               \n Using Python 3.6.0, PyQt5, pytvmaze, sqllite3, prettytable")

    def reset_screen(self):
        """ reset button clicked"""
        self.ScreenStatus = 0
        self.listinfo.clear()
        self.textEnter.clear()
        self.mainlbl.setText("Information Display")
        self.OneBtn.setEnabled(True)
        self.TwoBtn.setEnabled(False)
        self.FourBtn.setEnabled(True)
        self.FiveBtn.setEnabled(False)
        self.start_screen()

    def start_screen(self):
        """initialise start screen list box"""
        self.listinfo.addItem("Welcome: ")
        self.listinfo.addItem("\nWrite a tv show name in Input box and then press Enter to search Tv Series")
        self.listinfo.addItem("\nPress View Favs for Favourites")
        self.listinfo.addItem("\nPress Reset to reset screen")

    def one_button_click(self):
        """ button one ENTER clicked method"""
        self.FourBtn.setEnabled(False)
        # Search show, screenstatus =0, by inputing text
        if self.ScreenStatus == 0:
            if self.textEnter.text():
                self.listinfo.clear()
                items = self.myTvApi.search_tv_series(self.textEnter.text())
                if not items:
                    self.listinfo.addItem("There Is No Matching Tv Show :-")
                else:
                    items = str(items)
                    self.listinfo.addItem(items)
                    self.textEnter.clear()
                    self.ScreenStatus = 1
                    self.listinfo.addItem("To pick a show, enter a row Number in Input Box and press Enter button :-")
                    self.mainlbl.setText("TV show selection screen")
                    return
            else:
                self.listinfo.addItem("Enter a TV show name in Input Box and press Enter button :-")

        # pick show, screenstatus 1, by selecting row number
        if self.ScreenStatus == 1:
            if self.textEnter.text():
                # get tv show
                items = self.myTvApi.show_tv_series(self.textEnter.text())
                if not items:
                    self.listinfo.addItem("Pick a correct row number :-")
                else:
                    self.listinfo.clear()
                    self.listinfo.addItem(str(items))
                    self.OneBtn.setEnabled(False)
                    self.TwoBtn.setEnabled(True)
                    self.FiveBtn.setEnabled(True)
                    self.mainlbl.setText("Tv show information screen")
                    self.listinfo.addItem("Pick an information option on right and then press Select :-")
                    self.listinfo.addItem("You can also toggle Favorite status by entering maze_id in "
                                          "in the Input  box and pressing Edit Favs :-")
            else:
                self.listinfo.addItem("Enter a row Number in Input Box and press Enter Button :-")

        return

    def two_button_click(self):
        """ method to control button 2 SELECT click"""
        self.textEnter.clear()
        self.listinfo.clear()
        self.FiveBtn.setEnabled(False)
        # Scan radio buttons to see if selected run the function for that selection
        # pass the row no of table to relevant TvApi function so it knows show
        if self.OverviewRbtn.isChecked():
            self.listinfo.addItem(str(self.myTvApi.overview(1)))
        elif self.ActorsRbtn.isChecked():
            self.listinfo.addItem(str(self.myTvApi.actors(0)))
        elif self.seasonRbtn.isChecked():
            self.listinfo.addItem(str(self.myTvApi.seasons(0)))
        elif self.crewsRbtn.isChecked():
            self.listinfo.addItem(str(self.myTvApi.crews(0)))
        elif self.infoRbtn.isChecked():
            self.listinfo.addItem(str(self.myTvApi.more_info(1)))
        elif self.episodeRbtn.isChecked():
            self.listinfo.addItem(str(self.myTvApi.episode(0)))
        else:
            self.listinfo.addItem("Select an option from side bar:-")

    def three_button_click(self):
        """ method to control button 3 UPCOMING click """
        # TODO
        self.textEnter.clear()
        self.listinfo.clear()
        self.listinfo.addItem("This functionality not available yet :-")

    def four_button_click(self):
        """ method to control button 4 FAVOURITES click """
        self.FiveBtn.setEnabled(True)
        self.FourBtn.setEnabled(False)
        self.OneBtn.setEnabled(False)
        self.listinfo.clear()
        self.ScreenStatus = 0
        items = self.myTvApi.favour_show(self.myTvSql.display_db())
        self.listinfo.addItem(str(items))
        self.listinfo.addItem("Enter a maze_ID and press Edit Favs button, If present in database")
        self.listinfo.addItem("it will be Deleted, if not present it will be added :- ")
        self.mainlbl.setText("Favourites Information Screen")

    def five_button_click(self):
        """ method to control button 5 EDIT FAVS click"""
        self.OneBtn.setEnabled(False)
        self.TwoBtn.setEnabled(False)
        mazeid = self.textEnter.text()
        if mazeid:
            # check if in database
            showflag = self.myTvSql.scan_db(mazeid)
            if showflag:
                # True delete if in database
                self.myTvSql.del_db(mazeid)
                self.four_button_click()
                self.listinfo.addItem("Removed maze_id {} from the database:-".format(mazeid))
            else:
                # False add if not in database
                # check here if valid maze ID from API
                name = self.myTvApi.showname_get(mazeid)
                if name != "Error":
                    self.myTvSql.add_db(mazeid, name)
                    self.four_button_click()
                    self.listinfo.addItem("Added maze_id {} to the database:-".format(mazeid))
                else:
                    self.listinfo.addItem("Cannot find that Maze_ID with API :-")
        else:
            self.listinfo.addItem("Add a Maze_Id and press again :-")


# =====================MAIN===============================
def test(text):
    """ Test module imported function """
    logger.info(text)


if __name__ == '__main__':
    test("main")
else:
    test(" Imported {}".format(__name__))
# =====================END===============================



#!/usr/bin/env python3
""" This module is called from tv_viewer.py to handle Sqllite
database interaction, contains one class TvSqLight , 6 methods"""
# =========================MODULE HEADER=======================================
# title             :tv_sqllite.py
# description       :This module is called from tv_viewer.py to handle Sqlite interaction
# author            :Gavin Lyons
# date              :24/11/2017
# web               :https://github.com/gavinlyonsrepo/tv_viewer
# mail              :glyons66@hotmail.com
# python_version    :3.6.0

# ==========================IMPORTS======================
# Import the system modules needed to run
import os
import sqlite3


from logger_conf import logger_conf as myLog

# ====================== GLOBALS =======================
# setup logging
logger = myLog.my_logging(__name__)
DESTCONFIG = os.environ['HOME'] + "/.config/tv_viewer"
if not os.path.exists(DESTCONFIG):
    os.makedirs(DESTCONFIG)

# ====================CLASS SECTION===============================


class TvSqLight(object):
    """class to interact with sqlite database contains 6 methods, create close
    the database. add, delete, scan, and display data. object is initialise with a path"""
    # path for db to hold favs

    def __init__(self, name):
        self.name = name
        self.connection = ""
        self.path = DESTCONFIG + "/" + "fav.db"

    def create_db(self):
        """ Method to create the database one table two fields"""
        try:
            conn = sqlite3.connect(self.path)
            self.connection = conn.cursor()
            self.connection.execute("""CREATE TABLE IF NOT EXISTS shows (
                    number integer,
                    name text
                    )""")
        except sqlite3.OperationalError:
            logger.exception(" Failed to Create database: ")

    def scan_db(self, mazeid):
        """Method to scan scan database for an ID, return true if it exists, False if not"""
        try:
            flag = True
            conn = sqlite3.connect(self.path)
            self.connection = conn.cursor()
            self.connection.execute("SELECT * FROM shows WHERE number=:number", {'number': mazeid})
            if not self.connection.fetchall():
                # empty list if not in database
                flag = False
        except sqlite3.Error:
            flag = False
            logger.exception(" Failed to Scan database: ")
        return flag

    def display_db(self):
        """ method to display database called from Favs screen"""
        try:
            conn = sqlite3.connect(self.path)
            self.connection = conn.cursor()
            self.connection.execute("SELECT * FROM shows")
        except sqlite3.Error:
            logger.exception(" Failed to Display database: ")
        return self.connection.fetchall()

    def add_db(self, mazeid, showname):
        """ method to add record to database called by Fav edit button"""
        try:
            conn = sqlite3.connect(self.path)
            self.connection = conn.cursor()
            with conn:
                self.connection.execute("INSERT INTO shows VALUES (:number, :name)", {'number': mazeid, 'name': str(showname)})
        except sqlite3.Error:
            logger.exception(" Failed to Add record database: ")
        return

    def del_db(self, maze_id):
        """ method to delete record to database called by Fav edit button"""
        try:
            conn = sqlite3.connect(self.path)
            self.connection = conn.cursor()
            with conn:
                self.connection.execute("DELETE FROM shows WHERE number=:number", {'number': maze_id})
        except sqlite3.Error:
            logger.exception(" Failed to remove record database: ")
        return

    def close_db(self):
        """method to close database connection at end of program"""
        try:
            conn = sqlite3.connect(self.path)
            self.connection = conn.cursor()
            self.connection.close()
        except sqlite3.Error:
            logger.exception(" Failed to close database: ")
        return
# =====================MAIN===============================


def test(text):
    """ testing imported """
    logger.info(text)


if __name__ == '__main__':
    test(" main")
else:
    test("   Imported {}".format(__name__))
# =====================END===============================



#!/usr/bin/env python3
""" tv_viewer : GUI to view tv program details using TV maze application
programming interface. Written in python 3 and PyQt 5, it also stores user favourites in an
SQLite database. It uses python module pytvmaze to interface with API
and python module prettytable to help display results."""
# =========================MAIN SCRIPT HEADER=======================================
# title             :tv_viewer
# description       :Python PyQt5 GUI to view tv program details using tvmaze API and SQllite3 database
# author            :Gavin Lyons
# date              :20/08/2017
# version           :1.1
# web               :https://github.com/gavinlyonsrepo/tv_viewer
# mail              :glyons66@hotmail.com
# python_version    :3.6.0
# main api:         :http://www.tvmaze.com/api

# ==========================IMPORTS======================
# Import the system modules needed to run
import sys
from PyQt5 import QtWidgets
from PyQt5.QtGui import QIcon


# My modules
from tv_qt_class import tv_qt_class as myqt
from logger_conf import logger_conf as myLog

# =====================MAIN===============================
if __name__ == "__main__":
    logger = myLog.my_logging(__name__)
    logger.info("  Main Loop Start")
    app = QtWidgets.QApplication(sys.argv)
    if sys.platform == 'win32':
    	pass
    else:
    	app.setWindowIcon(QIcon('/usr/share/pixmaps/tv_viewer.png'))
    MainWindow = QtWidgets.QMainWindow()
    ui = myqt.Ui_MainWindow()
    ui.setupUi(MainWindow)
    MainWindow.show()
    sys.exit(app.exec_())
# =====================END===============================




from setuptools import setup
from sys import platform

if platform == 'win32':
    setup(
        name="tv_viewer",
        version="1.2",
        author="Gavin Lyons",
        author_email="glyons66@hotmail.com",
        description="Python PyQt5 GUI to view tv program details using tvmaze API and SQLite",
        license="GPL",
        keywords="televison tv_viewer tv maze SQLite PyQt gavin lyons schedule API GUI",
        url="https://github.com/gavinlyonsrepo/tv_viewer",
        download_url='https://github.com/gavinlyonsrepo/tv_viewer/archive/1.2.tar.gz',
        packages=['tv_viewer','tv_qt_class','tv_sqllite','tv_api_work','logger_conf'],
        install_requires=['pip','pytvmaze','prettytable',],
        setup_requires=['pip'],
        scripts=['tv_viewer/tv_viewer.py'],
        classifiers=[
            "Programming Language :: Python :: 3.6",
            "License :: OSI Approved :: GNU General Public License v3 (GPLv3)",
        ],
        )
else:
    setup(
        name="tv_viewer",
        version="1.2",
        author="Gavin Lyons",
        author_email="glyons66@hotmail.com",
        description="Python PyQt5 GUI to view tv program details using tvmaze API and SQLite",
        license="GPL",
        keywords="televison tv_viewer tv maze SQLite PyQt gavin lyons schedule API GUI",
        url="https://github.com/gavinlyonsrepo/tv_viewer",
        download_url='https://github.com/gavinlyonsrepo/tv_viewer/archive/1.2.tar.gz',
        packages=['tv_viewer','tv_qt_class','tv_sqllite','tv_api_work','logger_conf'],
        data_files=[('/usr/share/doc/tv_viewer/', ['README.md']),
                     ('/usr/share/pixmaps/', ['desktop/tv_viewer.png']),
                     ('/usr/share/applications/', ['desktop/tv_viewer.desktop'])],
        install_requires=['pip','pytvmaze','prettytable',],
        setup_requires=['pip'],
        scripts=['tv_viewer/tv_viewer.py'],
        classifiers=[
            "Programming Language :: Python :: 3.6",
            "License :: OSI Approved :: GNU General Public License v3 (GPLv3)",
        ], 
        )



#!/usr/bin/env python3
"""logger_conf.py module used by tv_viewer program, imported into modules sets up logging"""


# ==========================IMPORTS======================
# Import the system modules needed to run 
import logging 
import os
import datetime

# ====================FUNCTION SECTION===============================


def my_logging(module_name):
    DESTCACHE = os.environ['HOME'] + "/.cache/tv_viewer"
    if not os.path.exists(DESTCACHE):
        os.makedirs(DESTCACHE)
    logger = logging.getLogger(module_name)
    logger.setLevel(logging.INFO)
    basename = "tvviewerlog"
    suffix = datetime.datetime.now().strftime("%y%m%d_%H%M%S")
    filename = "_".join([basename, suffix]) # e.g. 'tvviewer_120508_171442'
    formatter = logging.Formatter('%(asctime)s:%(levelname)s:%(name)s:%(message)s')
    file_handler = logging.FileHandler(DESTCACHE + "/" + filename)
    file_handler.setLevel(logging.INFO)
    file_handler.setFormatter(formatter)
    logger.addHandler(file_handler)
    return logger

# =====================MAIN===============================


def test(text):
    """ testing function 1"""
    logger = my_logging(__name__)
    logger.info(text)


if __name__ == '__main__':
    test("main")
else:
    test(" Imported {}".format(__name__))
# =====================END===============================



#!/usr/bin/env python3
"""tv_api_work.py , Module called by TV_viewer contains a class(TvApi) to handle the tv maze API """
# ========================= MODULE HEADER=======================================
# title             :tv_api_work.py
# description       :module called by TV_viewer contains a class to handle the API
# author            :Gavin Lyons
# date              :24/11/2017
# web               :https://github.com/gavinlyonsrepo/tv_viewer
# mail              :glyons66@hotmail.com
# python_version    :3.6.0
# Api               :http://www.tvmaze.com/api


# ==========================IMPORTS======================
# Import the system modules needed to run

from datetime import datetime
import pytvmaze
from prettytable import PrettyTable
# my modules
from logger_conf import logger_conf as myLog
from tv_sqllite import tv_sqllite as mySql

# ==========================GLOBALS===============================
# setup logging
logger = myLog.my_logging(__name__)

# ==================== ClASS SECTION===============================


class TvApi(object):
    """ Class to handle interaction with tv maze API called by python program tv_viewer"""

    def __init__(self, name):
        self.tvshowtable = ""  # used to display list of shows
        self.tvshow2table = ""   # used to display show info
        self.name = name
        self.tvn = pytvmaze.TVMaze()
        # define instance of the SQl class for favourate view
        self.myTvSql = mySql.TvSqLight("mysqlobject1")

    def search_tv_series(self, itemshow):
        """Search for a Tv show, program status S1, takes in input text from user,
        Return table of finds or None"""
        try:
            items = pytvmaze.show_search(itemshow)
            self.tvshowtable = PrettyTable(vertical_char=" ")
            for i, item in enumerate(items):
                self.tvshowtable.field_names = [i+1, "seriesname"]
                self.tvshowtable.add_row([i, item])
        except pytvmaze.BaseError:
            logger.exception("Failure to find tv series")
            self.tvshowtable = None
        return self.tvshowtable

    def show_tv_series(self, row_no):
        """method to Select a Tv show, program status S2, passed row number user selected ,
        returns datatable of extracted info or None"""

        try:
            row_no = int(row_no)
            # get name of show from row number
            tv_show_name = self.tvshowtable.get_string(border=False, header=False,
                                                       start=row_no, end=(row_no + 1),
                                                       fields=["seriesname"])
            # put tv data in new table
            show = self.tvn.get_show(show_name=tv_show_name)
            delta_date, next_release_date = self.tv_maze_date(tv_show_name)
            self.tvshow2table = PrettyTable(vertical_char=" ")
            self.tvshow2table.field_names = ["Key", "Value"]
            self.tvshow2table.add_row(["Maze_id", show.maze_id])
            self.tvshow2table.add_row(["Name", show.name])
            self.tvshow2table.add_row(["Rating", show.rating])
            self.tvshow2table.add_row(["Genre", show.genres])
            self.tvshow2table.add_row(["Status", show.status])
            self.tvshow2table.add_row(["Premiered ", show.premiered])
            self.tvshow2table.add_row(["Last episode", show.previous_episode.airdate])
            self.tvshow2table.add_row(["Next episode", next_release_date])
            self.tvshow2table.add_row(["Season / epi",
                                       str(show.previous_episode.season_number)
                                       + "/" + str(show.previous_episode.episode_number)])
            self.tvshow2table.add_row(["days to next", delta_date])
            # scan the database to see if its a fav
            show_flag = self.myTvSql.scan_db(show.maze_id)
            if show_flag:
                # True its in favs database
                self.tvshow2table.add_row(["Favourite", "TRUE"])
            else:
                # false its not in favs database
                self.tvshow2table.add_row(["Favourite", "FALSE"])
        except Exception:
            logger.exception("Failure to extract info for TV show")
            self.tvshow2table = None
        return self.tvshow2table

    def showname_get(self, mazeid):
        """method to calculate if valid maze_ID in API called from fav_edit ,
        takes in a maze ID returns a string with showname or error"""
        try:
            showname = self.tvn.get_show(maze_id=mazeid)
        except Exception:
            logger.exception("Failure to find maze_ID")
            showname = "Error"
        return showname

    def favour_show(self, database):
        """mtd to display favs takes in list of tuple pairs
        from db returns the new data_table for display"""
        data_table = []
        try:
            data_table = PrettyTable(vertical_char=" ", padding_width=5)
            for num, name in database:
                delta_date, next_release_date = self.tv_maze_date(name)
                data_table.field_names = ["Days till", "Next epi", "Maze_id", "series" ]
                data_table.add_row([delta_date, next_release_date, num, name, ])
        except Exception:
            logger.exception("Failure to show favs")
        return data_table

    def tv_maze_date(self, showname):
        """ method to calculate days to next episode takes in showname
        returns days to next episode  and release Date"""
        try:
            show = self.tvn.get_show(show_name=showname)
            current_date = datetime.strptime(datetime.now().strftime('%Y-%m-%d'), '%Y-%m-%d')
            next_release_date = datetime.strptime(show.next_episode.airdate, '%Y-%m-%d').date()
            delta_date = (next_release_date - current_date.date()).days
        except Exception:
            logger.warning("No next episode data available")
            next_release_date = "n/a\t"
            delta_date = "n/a"
        return delta_date, next_release_date

    def overview(self, row_no):
        """ method to get overview of show passed in row number  returns the summary """
        summary = ""
        try:
            row_no = int(row_no)
            # get name of show from row number passed
            tvshowname = self.tvshow2table.get_string(border=False, header=False,
                                                      start=row_no, end=(row_no + 1),
                                                      fields=["Value"])
            show = self.tvn.get_show(show_name=tvshowname)
            summary = str(show.summary)
        except Exception:
            logger.exception("Failure to get summary")
        return summary

    def seasons(self, row_no):
        """ method to get seasons of show passed in rowno returns the seasons"""
        season_table = ""
        try:
            row_no = int(row_no)
            # get number of show from row number passed
            mazeid = self.tvshow2table.get_string(border=False, header=False, start=row_no,
                                                  end=(row_no + 1),
                                                  fields=["Value"])

            seasons = pytvmaze.show_seasons(int(mazeid))
            season_table = PrettyTable(vertical_char=" ")
            for i, item in enumerate(seasons):
                season_table.field_names = [i+1, "season"]
                season_table.add_row([i, item])
        except Exception:
            logger.exception("Failure to get seasons")
        return season_table

    def episode(self, row_no):
        """ method to get episode list of show passed
        in row no of table , returns the episodes in a table """
        epi_table = ""
        try:
            row_no = int(row_no)
            # get number of show from row number passed
            maze_id = self.tvshow2table.get_string(border=False, header=False, start=row_no,
                                                   end=(row_no + 1), fields=["Value"])

            episode = pytvmaze.episode_list(int(maze_id), specials=None)
            epi_table = PrettyTable(vertical_char=" ")
            for i, item in enumerate(episode):
                epi_table.field_names = [i+1, "episode"]
                epi_table.add_row([i, item])
        except Exception:
            logger.exception("Failure to get episodes")
        return epi_table

    def actors(self, row_no):
        """ method to get actors info of show passed in row no of table ,
        returns the info """
        actor_table = ""
        try:
            row_no = int(row_no)
            # get number of show from row number passed
            mazeid = self.tvshow2table.get_string(border=False, header=False, start=row_no,
                                                  end=(row_no + 1), fields=["Value"])

            mycast = pytvmaze.show_cast(int(mazeid)).people
            actor_table = PrettyTable(vertical_char=" ")
            for i, people in enumerate(mycast):
                actor_table.field_names = [i, "Character", "Person"]
                actor_table.add_row([i, people.character.name, people.name])
        except Exception:
            logger.exception("Failure to get actors")
        return actor_table

    def crews(self, row_no):
        """ method to get crews info of show passed in rowno returns the info """
        crew_table = ""
        try:
            row_no = int(row_no)
            # get number of show from row number passed
            mazeid = self.tvshow2table.get_string(border=False, header=False, start=row_no,
                                                  end=(row_no + 1), fields=["Value"])

            mycrew = pytvmaze.get_show_crew(int(mazeid))
            crew_table = PrettyTable(vertical_char=" ")
            for i, people in enumerate(mycrew):
                crew_table.field_names = [i, "Person", "Role"]
                crew_table.add_row([i, people.person.name, people.type])
        except Exception:
            logger.exception("Failure to get crews")
        return crew_table

    def more_info(self, row_no):
        """extracts tv show name from tv series table for a  given row number
        called from tvshowseries returns a datatable with show details"""
        data_table = ""
        tv_show_name = ""
        try:
            row_no = int(row_no)
            # get name of show from row number
            tv_show_name = self.tvshow2table.get_string(border=False, header=False,
                                                        start=row_no, end=(row_no + 1),
                                                        fields=["Value"])
            # put tv data in table
            show = self.tvn.get_show(show_name=tv_show_name)
            data_table = PrettyTable(vertical_char=" ")
            data_table.field_names = ["Key", "Value"]
            data_table.add_row(["Network", show.network.name])
            data_table.add_row(["Country", show.network.country])
            data_table.add_row(["Schedule Time", show.schedule['time']])
            data_table.add_row(["Schedule Days", show.schedule['days']])
            data_table.add_row(["Runtime", show.runtime])
        except Exception:
            logger.exception("Failure to get more info")
        return data_table


# =====================MAIN===============================


def test(text):
    """ test import function"""
    logger.info(text)


if __name__ == '__main__':
    test("main")
else:
    test(" Imported {}".format(__name__))
# =====================END===============================
